"use strict";

exports.__esModule = true;
exports.useContextSelector = void 0;
var React = _interopRequireWildcard(require("react"));
var _utils = require("./utils");
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
/**
 * This hook returns context selected value by selector.
 * It will only accept context created by `createContext`.
 * It will trigger re-render if only the selected value is referencially changed.
 */
var useContextSelector = function useContextSelector(context, selector) {
  var contextValue = React.useContext(context);
  var value = contextValue.value.current,
    version = contextValue.version.current,
    listeners = contextValue.listeners;
  var selected = selector(value);
  var _React$useReducer = React.useReducer(function (prevState, payload // from provider effect
    ) {
      if (!payload) {
        // early bail out when is dispatched during render
        return [value, selected];
      }
      if (payload[0] <= version) {
        if (Object.is(prevState[1], selected)) {
          return prevState; // bail out
        }

        return [value, selected];
      }
      try {
        if (Object.is(prevState[0], payload[1])) {
          return prevState; // do not update
        }

        var nextSelected = selector(payload[1]);
        if (Object.is(prevState[1], nextSelected)) {
          return prevState; // do not update
        }

        return [payload[1], nextSelected];
      } catch (e) {
        // ignored (stale props or some other reason)
      }
      return [].concat(prevState); // schedule update
    }, [value, selected]),
    state = _React$useReducer[0],
    dispatch = _React$useReducer[1];
  if (!Object.is(state[1], selected)) {
    // schedule re-render
    // this is safe because it's self contained
    dispatch(undefined);
  }
  (0, _utils.useIsomorphicLayoutEffect)(function () {
    listeners.push(dispatch);
    return function () {
      var index = listeners.indexOf(dispatch);
      listeners.splice(index, 1);
    };
  }, [listeners]);
  return state[1];
};
exports.useContextSelector = useContextSelector;
//# sourceMappingURL=useContextSelector.js.map
