"use strict";

exports.__esModule = true;
exports.DATA_IS_SCROLLABLE_ATTRIBUTE = void 0;
exports.findScrollableParent = findScrollableParent;
var _getDocument = require("./getDocument");
var _getWindow = require("./getWindow");
/**
 * Placing this attribute on scrollable divs optimizes detection to know
 * if the div is scrollable or not (given we can avoid expensive operations
 * like getComputedStyle.)
 *
 * @public
 */
var DATA_IS_SCROLLABLE_ATTRIBUTE = 'data-is-scrollable';

/**
 * Traverses up the DOM for the element with the data-is-scrollable=true attribute, or returns
 * document.body.
 *
 * @public
 */
exports.DATA_IS_SCROLLABLE_ATTRIBUTE = DATA_IS_SCROLLABLE_ATTRIBUTE;
function findScrollableParent(startingElement) {
  var el = startingElement;
  var doc = (0, _getDocument.getDocument)(startingElement);

  // First do a quick scan for the scrollable attribute.
  while (el && el !== doc.body) {
    if (el.getAttribute(DATA_IS_SCROLLABLE_ATTRIBUTE) === 'true') {
      return el;
    }
    el = el.parentElement;
  }

  // If we haven't found it, the use the slower method: compute styles to evaluate if overflow is set.
  el = startingElement;
  while (el && el !== doc.body) {
    if (el.getAttribute(DATA_IS_SCROLLABLE_ATTRIBUTE) !== 'false') {
      var computedStyles = getComputedStyle(el);
      var overflowY = computedStyles ? computedStyles.getPropertyValue('overflow-y') : '';
      if (overflowY && (overflowY === 'scroll' || overflowY === 'auto')) {
        return el;
      }
    }
    el = el.parentElement;
  }

  // Fall back to window scroll.
  if (!el || el === doc.body) {
    el = (0, _getWindow.getWindow)(startingElement);
  }
  return el;
}
//# sourceMappingURL=findScrollableParent.js.map
